@@ -65,7 +65,7 @@
 #[allow(unused_variables)]
 
 /// Wraps a parser in a closure
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! closure (
     ($ty:ty, $submac:ident!( $($args:tt)* )) => (
         |i: $ty| { $submac!(i, $($args)*) }
@@ -89,10 +89,12 @@ macro_rules! closure (
 /// // will use &[u8] as input type (use this if the compiler
 /// // complains about lifetime issues
 /// named!(my_function<&[u8]>,            tag!("abcd"));
-/// //prefix them with 'pub' to make the functions public
+/// // prefix them with 'pub' to make the functions public
 /// named!(pub my_function,               tag!("abcd"));
+/// // prefix them with 'pub(crate)' to make the functions public within the crate
+/// named!(pub(crate) my_function,               tag!("abcd"));
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! named (
     (#$($args:tt)*) => (
         named_attr!(#$($args)*);
@@ -147,10 +149,67 @@ macro_rules! named (
             $submac!(i, $($args)*)
         }
     );
+    (pub(crate) $name:ident( $i:ty ) -> $o:ty, $submac:ident!( $($args:tt)* )) => (
+        #[allow(unused_variables)]
+        pub(crate) fn $name( i: $i ) -> $crate::IResult<$i,$o, u32> {
+            $submac!(i, $($args)*)
+        }
+    );
+    (pub(crate) $name:ident<$i:ty,$o:ty,$e:ty>, $submac:ident!( $($args:tt)* )) => (
+        #[allow(unused_variables)]
+        pub(crate) fn $name( i: $i ) -> $crate::IResult<$i, $o, $e> {
+            $submac!(i, $($args)*)
+        }
+    );
+    (pub(crate) $name:ident<$i:ty,$o:ty>, $submac:ident!( $($args:tt)* )) => (
+        #[allow(unused_variables)]
+        pub(crate) fn $name( i: $i ) -> $crate::IResult<$i, $o, u32> {
+            $submac!(i, $($args)*)
+        }
+    );
+    (pub(crate) $name:ident<$o:ty>, $submac:ident!( $($args:tt)* )) => (
+        #[allow(unused_variables)]
+        pub(crate) fn $name( i: &[u8] ) -> $crate::IResult<&[u8], $o, u32> {
+            $submac!(i, $($args)*)
+        }
+    );
+    (pub(crate) $name:ident, $submac:ident!( $($args:tt)* )) => (
+        #[allow(unused_variables)]
+        pub(crate) fn $name<'a>( i: &'a [u8] ) -> $crate::IResult<&[u8], &[u8], u32> {
+            $submac!(i, $($args)*)
+        }
+    );
 );
 
 /// Makes a function from a parser combination with arguments.
-#[macro_export]
+///
+/// ```ignore
+/// //takes `&[u8]` as input
+/// named_args!(tagged(open_tag: &[u8], close_tag: &[u8])<&str>,
+///   delimited!(tag!(open_tag), map_res!(take!(4), str::from_utf8), tag!(close_tag))
+/// );
+
+/// //takes `&str` as input
+/// named_args!(tagged(open_tag: &str, close_tag: &str)<&str, &str>,
+///   delimited!(tag!(open_tag), take!(4), tag!(close_tag))
+/// );
+/// ```
+///
+/// Note: if using arguments that way gets hard to read, it is always
+/// possible to write the equivalent parser definition manually, like
+/// this:
+///
+/// ```ignore
+/// fn tagged(input: &[u8], open_tag: &[u8], close_tag: &[u8]) -> IResult<&[u8], &str> {
+///   // the first combinator in the tree gets the input as argument. It is then
+///   // passed from one combinator to the next through macro rewriting
+///   delimited!(input,
+///     tag!(open_tag), take!(4), tag!(close_tag)
+///   )
+/// );
+/// ```
+///
+#[macro_export(local_inner_macros)]
 macro_rules! named_args {
     (pub $func_name:ident ( $( $arg:ident : $typ:ty ),* ) < $return_type:ty > , $submac:ident!( $($args:tt)* ) ) => {
         pub fn $func_name(input: &[u8], $( $arg : $typ ),*) -> $crate::IResult<&[u8], $return_type> {
@@ -165,6 +224,16 @@ macro_rules! named_args {
           $submac!(input, $($args)*)
         }
     };
+    (pub(crate) $func_name:ident ( $( $arg:ident : $typ:ty ),* ) < $return_type:ty > , $submac:ident!( $($args:tt)* ) ) => {
+        pub(crate) fn $func_name(input: &[u8], $( $arg : $typ ),*) -> $crate::IResult<&[u8], $return_type> {
+            $submac!(input, $($args)*)
+        }
+    };
+    (pub(crate) $func_name:ident < 'a > ( $( $arg:ident : $typ:ty ),* ) < $return_type:ty > , $submac:ident!( $($args:tt)* ) ) => {
+        pub(crate) fn $func_name<'this_is_probably_unique_i_hope_please, 'a>(input: &'this_is_probably_unique_i_hope_please [u8], $( $arg : $typ ),*) -> $crate::IResult<&'this_is_probably_unique_i_hope_please [u8], $return_type> {
+            $submac!(input, $($args)*)
+        }
+    };
     ($func_name:ident ( $( $arg:ident : $typ:ty ),* ) < $return_type:ty > , $submac:ident!( $($args:tt)* ) ) => {
         fn $func_name(input: &[u8], $( $arg : $typ ),*) -> $crate::IResult<&[u8], $return_type> {
             $submac!(input, $($args)*)
@@ -220,7 +289,7 @@ macro_rules! named_args {
 /// // Multiple attributes can be passed if required
 /// named!(#[doc = "My Func"] #[inline(always)], pub my_function, tag!("abcd"));
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! named_attr (
     ($(#[$attr:meta])*, $name:ident( $i:ty ) -> $o:ty, $submac:ident!( $($args:tt)* )) => (
         $(#[$attr])*
@@ -282,6 +351,36 @@ macro_rules! named_attr (
             $submac!(i, $($args)*)
         }
     );
+    ($(#[$attr:meta])*, pub(crate) $name:ident( $i:ty ) -> $o:ty, $submac:ident!( $($args:tt)* )) => (
+        $(#[$attr])*
+        pub(crate) fn $name( i: $i ) -> $crate::IResult<$i,$o, u32> {
+            $submac!(i, $($args)*)
+        }
+    );
+    ($(#[$attr:meta])*, pub(crate) $name:ident<$i:ty,$o:ty,$e:ty>, $submac:ident!( $($args:tt)* )) => (
+        $(#[$attr])*
+        pub(crate) fn $name( i: $i ) -> $crate::IResult<$i, $o, $e> {
+            $submac!(i, $($args)*)
+        }
+    );
+    ($(#[$attr:meta])*, pub(crate) $name:ident<$i:ty,$o:ty>, $submac:ident!( $($args:tt)* )) => (
+        $(#[$attr])*
+        pub(crate) fn $name( i: $i ) -> $crate::IResult<$i, $o, u32> {
+            $submac!(i, $($args)*)
+        }
+    );
+    ($(#[$attr:meta])*, pub(crate) $name:ident<$o:ty>, $submac:ident!( $($args:tt)* )) => (
+        $(#[$attr])*
+        pub(crate) fn $name( i: &[u8] ) -> $crate::IResult<&[u8], $o, u32> {
+            $submac!(i, $($args)*)
+        }
+    );
+    ($(#[$attr:meta])*, pub(crate) $name:ident, $submac:ident!( $($args:tt)* )) => (
+        $(#[$attr])*
+        pub(crate) fn $name<'a>( i: &'a [u8] ) -> $crate::IResult<&[u8], &[u8], u32> {
+            $submac!(i, $($args)*)
+        }
+    );
 );
 
 /// Used to wrap common expressions and function as macros
@@ -296,7 +395,7 @@ macro_rules! named_attr (
 ///   named!(parser, call!(take_wrapper, 2));
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! call (
   ($i:expr, $fun:expr) => ( $fun( $i ) );
   ($i:expr, $fun:expr, $($args:expr),* ) => ( $fun( $i, $($args),* ) );
@@ -314,7 +413,7 @@ macro_rules! call (
 ///   named!(parser, apply!(take_wrapper, 2));
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! apply (
   ($i:expr, $fun:expr, $($args:expr),* ) => ( $fun( $i, $($args),* ) );
 );
@@ -367,12 +466,12 @@ macro_rules! apply (
 /// # }
 /// ```
 ///
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! return_error (
   ($i:expr, $code:expr, $submac:ident!( $($args:tt)* )) => (
     {
       use $crate::lib::std::result::Result::*;
-      use $crate::{Context,Err,ErrorKind};
+      use $crate::{Context,Err};
 
       let i_ = $i.clone();
       let cl = || {
@@ -397,15 +496,13 @@ macro_rules! return_error (
   ($i:expr, $submac:ident!( $($args:tt)* )) => (
     {
       use $crate::lib::std::result::Result::*;
-      use $crate::{Context,Err,ErrorKind};
+      use $crate::{Context,Err};
 
       let i_ = $i.clone();
       let cl = || {
         $submac!(i_, $($args)*)
       };
 
-      fn unify_types<I,E>(_: &Context<I,E>, _: &Context<I,E>) {}
-
       match cl() {
         Err(Err::Incomplete(x)) => Err(Err::Incomplete(x)),
         Ok((i, o))              => Ok((i, o)),
@@ -440,22 +537,19 @@ macro_rules! return_error (
 /// # }
 /// ```
 ///
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! add_return_error (
   ($i:expr, $code:expr, $submac:ident!( $($args:tt)* )) => (
     {
       use $crate::lib::std::result::Result::*;
-      use $crate::{Err,Context,ErrorKind};
+      use $crate::{Err,ErrorKind};
 
-      fn unify_types<I,E>(_: &Context<I,E>, _: &Context<I,E>) {}
       match $submac!($i, $($args)*) {
-        Ok((i, o))    => Ok((i, o)),
-        Err(Err::Error(e))      => {
-          unify_types(&e, &Context::Code($i, $code));
+        Ok((i, o)) => Ok((i, o)),
+        Err(Err::Error(e)) => {
           Err(Err::Error(error_node_position!($i, $code, e)))
         },
-        Err(Err::Failure(e))      => {
-          unify_types(&e, &Context::Code($i, $code));
+        Err(Err::Failure(e)) => {
           Err(Err::Failure(error_node_position!($i, $code, e)))
         },
         Err(e) => Err(e),
@@ -484,7 +578,7 @@ macro_rules! add_return_error (
 /// # }
 /// ```
 ///
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! complete (
   ($i:expr, $submac:ident!( $($args:tt)* )) => (
     {
@@ -534,7 +628,7 @@ macro_rules! complete (
 /// assert_eq!(r1, Err(Err::Error(error_position!(&[2,3,4,5][..], ErrorKind::ExprOpt))));
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! try_parse (
   ($i:expr, $submac:ident!( $($args:tt)* )) => ({
     use $crate::lib::std::result::Result::*;
@@ -551,7 +645,7 @@ macro_rules! try_parse (
 
 /// `map!(I -> IResult<I,O>, O -> P) => I -> IResult<I, P>`
 /// maps a function on the result of a parser
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! map(
   // Internal parser, do not use directly
   (__impl $i:expr, $submac:ident!( $($args:tt)* ), $g:expr) => (
@@ -575,7 +669,7 @@ macro_rules! map(
 
 /// `map_res!(I -> IResult<I,O>, O -> Result<P>) => I -> IResult<I, P>`
 /// maps a function returning a Result on the output of a parser
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! map_res (
   // Internal parser, do not use directly
   (__impl $i:expr, $submac:ident!( $($args:tt)* ), $submac2:ident!( $($args2:tt)* )) => (
@@ -612,9 +706,48 @@ macro_rules! map_res (
   );
 );
 
+/// `map_res_err!(I -> IResult<I,O>, O -> Result<P>) => I -> IResult<I, P>`
+/// maps a function returning a Result on the output of a parser, preserving the returned error
+#[macro_export(local_inner_macros)]
+macro_rules! map_res_err (
+  // Internal parser, do not use directly
+  (__impl $i:expr, $submac:ident!( $($args:tt)* ), $submac2:ident!( $($args2:tt)* )) => (
+    {
+      use $crate::lib::std::result::Result::*;
+      use $crate::{Context, Convert, Err};
+
+      let i_ = $i.clone();
+      match $submac!(i_, $($args)*) {
+        Ok((i,o)) => {
+          match $submac2!(o, $($args2)*) {
+            Ok(output) => Ok((i, output)),
+            Err(e) => {
+              let e = Context::convert(Context::Code($i, $crate::ErrorKind::Custom(e)));
+              Err(Err::Error(error_node_position!($i, $crate::ErrorKind::MapRes, e)))
+            },
+          }
+        },
+        Err(e) => Err(e),
+      }
+    }
+  );
+  ($i:expr, $submac:ident!( $($args:tt)* ), $g:expr) => (
+    map_res_err!(__impl $i, $submac!($($args)*), call!($g));
+  );
+  ($i:expr, $submac:ident!( $($args:tt)* ), $submac2:ident!( $($args2:tt)* )) => (
+    map_res_err!(__impl $i, $submac!($($args)*), $submac2!($($args2)*));
+  );
+  ($i:expr, $f:expr, $g:expr) => (
+    map_res_err!(__impl $i, call!($f), call!($g));
+  );
+  ($i:expr, $f:expr, $submac:ident!( $($args:tt)* )) => (
+    map_res_err!(__impl $i, call!($f), $submac!($($args)*));
+  );
+);
+
 /// `map_opt!(I -> IResult<I,O>, O -> Option<P>) => I -> IResult<I, P>`
 /// maps a function returning an Option on the output of a parser
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! map_opt (
   // Internal parser, do not use directly
   (__impl $i:expr, $submac:ident!( $($args:tt)* ), $submac2:ident!( $($args2:tt)* )) => (
@@ -655,7 +788,7 @@ macro_rules! map_opt (
 /// input to the specified type
 ///
 /// this will completely consume the input
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! parse_to (
   ($i:expr, $t:ty ) => (
     {
@@ -671,7 +804,7 @@ macro_rules! parse_to (
       let res: Option<$t> = ($i).parse_to();
       match res {
         Some(output) => Ok(($i.slice($i.input_len()..), output)),
-        None         => Err(Err::Error(Context::Code($i, ErrorKind::MapOpt::<u32>)))
+        None         => Err(Err::Error(Context::Code($i, ErrorKind::ParseTo::<u32>)))
       }
     }
   );
@@ -686,7 +819,7 @@ macro_rules! parse_to (
 ///  named!(check<u32>, verify!(nom::be_u32, |val:u32| val >= 0 && val < 3));
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! verify (
   // Internal parser, do not use directly
   (__impl $i:expr, $submac:ident!( $($args:tt)* ), $submac2:ident!( $($args2:tt)* )) => (
@@ -738,7 +871,7 @@ macro_rules! verify (
 ///  assert_eq!(r2, Ok((&b" aaa"[..], 42)));
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! value (
   ($i:expr, $res:expr, $submac:ident!( $($args:tt)* )) => (
     {
@@ -757,7 +890,8 @@ macro_rules! value (
   );
   ($i:expr, $res:expr) => (
     {
-      Ok(($i, $res))
+      let res: $crate::IResult<_,_> = Ok(($i, $res));
+      res
     }
   );
 );
@@ -766,7 +900,7 @@ macro_rules! value (
 /// evaluate an expression that returns a Result<T,E> and returns a Ok((I,T)) if Ok
 ///
 /// See expr_opt for an example
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! expr_res (
   ($i:expr, $e:expr) => (
     {
@@ -811,7 +945,7 @@ macro_rules! expr_res (
 /// assert_eq!(r1, Err(Err::Error(error_position!(&[2,3,4,5][..], ErrorKind::ExprOpt))));
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! expr_opt (
   ($i:expr, $e:expr) => (
     {
@@ -851,7 +985,7 @@ macro_rules! expr_opt (
 ///  assert_eq!(o(&b[..]), Ok((&b"bcdefg"[..], None)));
 ///  # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! opt(
   ($i:expr, $submac:ident!( $($args:tt)* )) => (
     {
@@ -891,7 +1025,7 @@ macro_rules! opt(
 ///  assert_eq!(o(&b[..]), Ok((&b"bcdefg"[..], Err(error_position!(&b[..], ErrorKind::Tag))));
 ///  # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! opt_res (
   ($i:expr, $submac:ident!( $($args:tt)* )) => (
     {
@@ -944,12 +1078,11 @@ macro_rules! opt_res (
 ///  # }
 /// ```
 ///
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! cond_with_error(
   ($i:expr, $cond:expr, $submac:ident!( $($args:tt)* )) => (
     {
       use $crate::lib::std::result::Result::*;
-      use $crate::{Convert,Err,Needed,IResult};
 
       if $cond {
         match $submac!($i, $($args)*) {
@@ -999,7 +1132,7 @@ macro_rules! cond_with_error(
 ///  # }
 /// ```
 ///
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! cond(
   ($i:expr, $cond:expr, $submac:ident!( $($args:tt)* )) => (
     {
@@ -1057,7 +1190,7 @@ macro_rules! cond(
 ///  # }
 /// ```
 ///
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! cond_reduce(
   ($i:expr, $cond:expr, $submac:ident!( $($args:tt)* )) => (
     {
@@ -1098,7 +1231,7 @@ macro_rules! cond_reduce(
 ///  assert_eq!(r, Ok((&b"abcdefgh"[..], &b"abcd"[..])));
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! peek(
   ($i:expr, $submac:ident!( $($args:tt)* )) => (
     {
@@ -1139,7 +1272,7 @@ macro_rules! peek(
 /// assert_eq!(r2, Err(Err::Error(error_position!(&b"e"[..], ErrorKind::Not))));
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! not(
   ($i:expr, $submac:ident!( $($args:tt)* )) => (
     {
@@ -1184,7 +1317,7 @@ macro_rules! not(
 ///  assert_eq!(r, Ok((&b"efgh"[..], &b"abcd"[..])));
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! tap (
   ($i:expr, $name:ident : $submac:ident!( $($args:tt)* ) => $e:expr) => (
     {
@@ -1216,7 +1349,7 @@ macro_rules! tap (
 /// will succeed
 ///
 /// TODO: example
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! eof (
   ($i:expr,) => (
     {
@@ -1241,7 +1374,7 @@ macro_rules! eof (
 /// more refills (like what happens when buffering big files).
 ///
 /// TODO: example
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! exact (
   ($i:expr, $submac:ident!( $($args:tt)* )) => ({
       terminated!($i, $submac!( $($args)*), eof!())
@@ -1262,7 +1395,7 @@ macro_rules! exact (
 ///  assert_eq!(r, Ok((&b" aaa"[..], &b"<!-- abc -->"[..])));
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! recognize (
   ($i:expr, $submac:ident!( $($args:tt)* )) => (
     {
@@ -1341,6 +1474,17 @@ mod tests {
     assert_eq!(res, Ok((&b""[..], a)));
   }
 
+  mod pub_crate_named_mod {
+    named!(pub(crate) tst, tag!("abcd"));
+  }
+
+  #[test]
+  fn pub_crate_named_test() {
+    let a = &b"abcd"[..];
+    let res = pub_crate_named_mod::tst(a);
+    assert_eq!(res, Ok((&b""[..], a)));
+  }
+
   #[test]
   fn apply_test() {
     fn sum2(a: u8, b: u8) -> u8 {
@@ -1516,4 +1660,79 @@ mod tests {
     );
     assert_eq!(test(&b"abcdefg"[..]), Ok((&b"fg"[..], &b"abcde"[..])));
   }
+
+  #[test]
+  fn parse_to() {
+    use util::Convert;
+
+    assert_eq!(
+      parse_to!("ab", usize),
+      Err(Err::Error(error_position!(
+        "ab",
+        ErrorKind::ParseTo
+      )))
+    );
+    assert_eq!(parse_to!("42", usize), Ok(("", 42)));
+    assert_eq!(ErrorKind::<u64>::convert(ErrorKind::ParseTo::<u32>), ErrorKind::ParseTo::<u64>);
+  }
+
+  #[test]
+  fn map_res_err() {
+    use Context;
+    use be_u8;
+
+    #[derive(Debug, Eq, PartialEq)]
+    enum ParseError {
+      InvalidValue(u8),
+    }
+
+    impl From<u32> for ParseError {
+      fn from(_: u32) -> Self {
+        unreachable!()
+      }
+    }
+
+    #[derive(Debug, Eq, PartialEq)]
+    enum ValidValue {
+      One,
+      Two,
+    }
+
+    fn validate(value: u8) -> Result<ValidValue, ParseError> {
+      match value {
+        b'1' => Ok(ValidValue::One),
+        b'2' => Ok(ValidValue::Two),
+        _ => Err(ParseError::InvalidValue(value))
+      }
+    }
+
+    named!(test<&[u8], ValidValue, ParseError>,
+      map_res_err!(fix_error!(ParseError, be_u8), validate)
+    );
+
+    assert_eq!(test(&b"1"[..]), Ok((&b""[..], ValidValue::One)));
+    assert_eq!(test(&b"2"[..]), Ok((&b""[..], ValidValue::Two)));
+
+    #[cfg(feature = "verbose-errors")]
+    {
+      assert_eq!(
+        test(&b"3"[..]),
+        Err(
+          Err::Error(
+            Context::List(
+              vec![
+              (&b"3"[..], ErrorKind::Custom(ParseError::InvalidValue(b'3'))),
+              (&b"3"[..], ErrorKind::MapRes)
+              ]
+            )
+          )
+        )
+      );
+    }
+
+    #[cfg(not(feature = "verbose-errors"))]
+    {
+      assert_eq!(test(&b"3"[..]), Err(Err::Error(Context::Code(&b"3"[..], ErrorKind::MapRes))));
+    }
+  }
 }
