@@ -87,6 +87,22 @@ impl HexDisplay for str {
   }
 }
 
+#[macro_export]
+macro_rules! nom_line (
+  () => (line!());
+);
+
+#[macro_export]
+macro_rules! nom_println (
+  ($($args:tt)*) => (println!($($args)*));
+);
+
+#[macro_export]
+macro_rules! nom_stringify (
+  ($($args:tt)*) => (stringify!($($args)*));
+);
+
+
 /// Prints a message if the parser fails
 ///
 /// The message prints the `Error` or `Incomplete`
@@ -104,15 +120,15 @@ impl HexDisplay for str {
 ///    f(a);
 /// # }
 /// ```
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! dbg (
   ($i: expr, $submac:ident!( $($args:tt)* )) => (
     {
       use $crate::lib::std::result::Result::*;
-      let l = line!();
+      let l = nom_line!();
       match $submac!($i, $($args)*) {
         Err(e) => {
-          println!("Err({:?}) at l.{} by ' {} '", e, l, stringify!($submac!($($args)*)));
+          nom_println!("Err({:?}) at l.{} by ' {} '", e, l, nom_stringify!($submac!($($args)*)));
           Err(e)
         },
         a => a,
@@ -144,15 +160,15 @@ macro_rules! dbg (
 ///    // 00000000        65 66 67 68 69 6a 6b 6c         efghijkl
 ///    f(a);
 /// # }
-#[macro_export]
+#[macro_export(local_inner_macros)]
 macro_rules! dbg_dmp (
   ($i: expr, $submac:ident!( $($args:tt)* )) => (
     {
       use $crate::HexDisplay;
-      let l = line!();
+      let l = nom_line!();
       match $submac!($i, $($args)*) {
         Err(e) => {
-          println!("Error({:?}) at l.{} by ' {} '\n{}", e, l, stringify!($submac!($($args)*)), $i.to_hex(8));
+          nom_println!("Error({:?}) at l.{} by ' {} '\n{}", e, l, nom_stringify!($submac!($($args)*)), $i.to_hex(8));
           Err(e)
         },
         a => a,
@@ -490,6 +506,10 @@ pub enum ErrorKind<E = u32> {
   TakeTill1,
   TakeUntilAndConsume1,
   TakeWhileMN,
+  ParseTo,
+  TooLarge,
+  Many0Count,
+  Many1Count,
 }
 
 #[cfg_attr(rustfmt, rustfmt_skip)]
@@ -557,6 +577,10 @@ pub fn error_to_u32<E>(e: &ErrorKind<E>) -> u32 {
     ErrorKind::TakeTill1                 => 67,
     ErrorKind::TakeUntilAndConsume1      => 68,
     ErrorKind::TakeWhileMN               => 69,
+    ErrorKind::ParseTo                   => 70,
+    ErrorKind::TooLarge                  => 71,
+    ErrorKind::Many0Count                => 72,
+    ErrorKind::Many1Count                => 73,
   }
 }
 
@@ -626,6 +650,10 @@ impl<E> ErrorKind<E> {
       ErrorKind::TakeTill1                 => "TakeTill1",
       ErrorKind::TakeUntilAndConsume1      => "Take at least 1 until and consume",
       ErrorKind::TakeWhileMN               => "TakeWhileMN",
+      ErrorKind::ParseTo                   => "Parse string to the specified type",
+      ErrorKind::TooLarge                  => "Needed data size is too large",
+      ErrorKind::Many0Count                => "Count occurrence of >=0 patterns",
+      ErrorKind::Many1Count                => "Count occurrence of >=1 patterns",
     }
   }
 
@@ -707,6 +735,10 @@ impl<F, E: From<F>> Convert<ErrorKind<F>> for ErrorKind<E> {
       ErrorKind::TakeTill1                 => ErrorKind::TakeTill1,
       ErrorKind::TakeUntilAndConsume1      => ErrorKind::TakeUntilAndConsume1,
       ErrorKind::TakeWhileMN               => ErrorKind::TakeWhileMN,
+      ErrorKind::ParseTo                   => ErrorKind::ParseTo,
+      ErrorKind::TooLarge                  => ErrorKind::TooLarge,
+      ErrorKind::Many0Count                => ErrorKind::Many0Count,
+      ErrorKind::Many1Count                => ErrorKind::Many1Count,
     }
   }
 }
