@@ -2,7 +2,6 @@
 //#![feature(trace_macros)]
 #![allow(dead_code)]
 #![allow(unused_comparisons)]
-#![allow(unused_doc_comment)]
 #![allow(unused_variables)]
 #![allow(unused_imports)]
 
@@ -37,9 +36,7 @@ named!(
 #[cfg(feature = "alloc")]
 fn wrap_suffix(input: &Option<Vec<&[u8]>>) -> Option<String> {
   if input.is_some() {
-    ///
-    /// I've tried both of the lines below individually and get the same error.
-    ///
+    // I've tried both of the lines below individually and get the same error.
     Some("hello".to_string())
   //Some(str::from_utf8(u).expect("Found invalid UTF-8").to_string())
   } else {
